#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"

#include "led.h"
#include "key.h"
#include "usart.h"

#define START_TASK_PRIO						1
#define START_STK_SIZE					120
void start_task(void *pvParameters);	//任务函数
TaskHandle_t StartTask_Handler;		//任务句柄

#define TASK1_TASK_PRIO						2
#define TASK1_STK_SIZE					120
void task1_task(void *pvParameters);
TaskHandle_t Task1Task_Handler;		//任务句柄

#define KEYPROCESS_TASK_PRIO						3
#define KEYPROCESS_STK_SIZE					120
void keyprocess_task(void *pvParameters);
TaskHandle_t KEYPROCESSTask_Handler;		//任务句柄

QueueHandle_t Key_Queue;		//按键队列

int main(void)
{
		LED_Init();
	  KEY_Init();
	  uart_init(72,115200);
		xTaskCreate((TaskFunction_t	) start_task, 
								(char *					)	"start_task", 
								(uint16_t				)	START_STK_SIZE,
								(void *					)	NULL,
								(UBaseType_t		)	START_TASK_PRIO,
								(TaskHandle_t *	)	&StartTask_Handler );
		vTaskStartScheduler();
}

void start_task(void *pvParameters)
{
	taskENTER_CRITICAL();		//进入临界区
	
	//创建队列
	Key_Queue = xQueueCreate(1,1);
	if(Key_Queue == NULL)
	{
		return ;
	}
	
	//创建task1
	xTaskCreate((TaskFunction_t	) task1_task, 
							(char *					)	"task1_task", 
							(uint16_t				)	TASK1_STK_SIZE,
							(void *					)	NULL,
							(UBaseType_t		)	TASK1_TASK_PRIO,
							(TaskHandle_t *	)	&Task1Task_Handler );
	//创建task2
	xTaskCreate((TaskFunction_t	) keyprocess_task, 
							(char *					)	"keyprocess_task", 
							(uint16_t				)	KEYPROCESS_STK_SIZE,
							(void *					)	NULL,
							(UBaseType_t		)	KEYPROCESS_TASK_PRIO,
							(TaskHandle_t *	)	&KEYPROCESSTask_Handler );
	
	vTaskDelete(StartTask_Handler);//删除自身也可以用NULL
	
	taskEXIT_CRITICAL();		//退出临界区
}

void task1_task(void *pvParameters)
{
	u8 key = 0;
	BaseType_t err;
	for(;;)
	{
		key = KEY_Scan(0);
		if((Key_Queue != NULL)&&(key))
		{
			err = xQueueSend(Key_Queue,&key,10); 
			if(err != pdTRUE)
			{
				return;
			}
		}
		printf("task1 running!\r\n");
		LED0 = ~LED0;
		vTaskDelay(10);
	}
}
void keyprocess_task(void *pvParameters)
{
	u8 key_buff;
	BaseType_t err;
	for(;;)
	{
		if(Key_Queue != NULL)
		{
			err = xQueueReceive(Key_Queue,&key_buff,portMAX_DELAY);
			if(err == pdTRUE)
			{
				if(key_buff)
				{
					LED1 = ~LED1;

					USART1->DR = 'b';
					while((USART1->SR&0X40)==0);//等待发送结束

					key_buff = 0;
				}
			}
		}
		else{
			vTaskDelay(100);
		}
	}
}

